@@ -4,6 +4,9 @@ mod system_error;
 mod verification_error;
 
 pub use recover_pubkey_error::RecoverPubkeyError;
-pub use std_error::{DivideByZeroError, OverflowError, OverflowOperation, StdError, StdResult};
+pub use std_error::{
+    CheckedFromRatioError, CheckedMultiplyRatioError, ConversionOverflowError, DivideByZeroError,
+    OverflowError, OverflowOperation, StdError, StdResult,
+};
 pub use system_error::SystemError;
 pub use verification_error::VerificationError;
