@@ -5,8 +5,9 @@
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 use std::cmp::{Ord, Ordering, PartialOrd};
-use std::fmt;
 
+#[cfg(feature = "ibc3")]
+use crate::addresses::Addr;
 use crate::binary::Binary;
 use crate::coins::Coin;
 use crate::errors::StdResult;
@@ -120,6 +121,7 @@ pub struct IbcChannel {
     pub endpoint: IbcEndpoint,
     pub counterparty_endpoint: IbcEndpoint,
     pub order: IbcOrder,
+    /// Note: in ibcv3 this may be "", in the IbcOpenChannel handshake messages
     pub version: String,
     /// The connection upon which this channel was created. If this is a multi-hop
     /// channel, we only expose the first hop.
@@ -297,6 +299,19 @@ impl From<IbcChannelOpenMsg> for IbcChannel {
     }
 }
 
+/// Note that this serializes as "null".
+#[cfg(not(feature = "ibc3"))]
+pub type IbcChannelOpenResponse = ();
+/// This serializes either as "null" or a JSON object.
+#[cfg(feature = "ibc3")]
+pub type IbcChannelOpenResponse = Option<Ibc3ChannelOpenResponse>;
+
+#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
+pub struct Ibc3ChannelOpenResponse {
+    /// We can set the channel version to a different one than we were called with
+    pub version: String,
+}
+
 /// The message that is passed into `ibc_channel_connect`
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
 #[serde(rename_all = "snake_case")]
@@ -392,12 +407,20 @@ impl From<IbcChannelCloseMsg> for IbcChannel {
 #[non_exhaustive]
 pub struct IbcPacketReceiveMsg {
     pub packet: IbcPacket,
+    #[cfg(feature = "ibc3")]
+    pub relayer: Addr,
 }
 
 impl IbcPacketReceiveMsg {
+    #[cfg(not(feature = "ibc3"))]
     pub fn new(packet: IbcPacket) -> Self {
         Self { packet }
     }
+
+    #[cfg(feature = "ibc3")]
+    pub fn new(packet: IbcPacket, relayer: Addr) -> Self {
+        Self { packet, relayer }
+    }
 }
 
 /// The message that is passed into `ibc_packet_ack`
@@ -406,15 +429,31 @@ impl IbcPacketReceiveMsg {
 pub struct IbcPacketAckMsg {
     pub acknowledgement: IbcAcknowledgement,
     pub original_packet: IbcPacket,
+    #[cfg(feature = "ibc3")]
+    pub relayer: Addr,
 }
 
 impl IbcPacketAckMsg {
+    #[cfg(not(feature = "ibc3"))]
     pub fn new(acknowledgement: IbcAcknowledgement, original_packet: IbcPacket) -> Self {
         Self {
             acknowledgement,
             original_packet,
         }
     }
+
+    #[cfg(feature = "ibc3")]
+    pub fn new(
+        acknowledgement: IbcAcknowledgement,
+        original_packet: IbcPacket,
+        relayer: Addr,
+    ) -> Self {
+        Self {
+            acknowledgement,
+            original_packet,
+            relayer,
+        }
+    }
 }
 
 /// The message that is passed into `ibc_packet_timeout`
@@ -422,12 +461,20 @@ impl IbcPacketAckMsg {
 #[non_exhaustive]
 pub struct IbcPacketTimeoutMsg {
     pub packet: IbcPacket,
+    #[cfg(feature = "ibc3")]
+    pub relayer: Addr,
 }
 
 impl IbcPacketTimeoutMsg {
+    #[cfg(not(feature = "ibc3"))]
     pub fn new(packet: IbcPacket) -> Self {
         Self { packet }
     }
+
+    #[cfg(feature = "ibc3")]
+    pub fn new(packet: IbcPacket, relayer: Addr) -> Self {
+        Self { packet, relayer }
+    }
 }
 
 /// This is the return value for the majority of the ibc handlers.
@@ -439,10 +486,7 @@ impl IbcPacketTimeoutMsg {
 /// will use other Response types
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
 #[non_exhaustive]
-pub struct IbcBasicResponse<T = Empty>
-where
-    T: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+pub struct IbcBasicResponse<T = Empty> {
     /// Optional list of messages to pass. These will be executed in order.
     /// If the ReplyOn member is set, they will invoke this contract's `reply` entry point
     /// after execution. Otherwise, they act like "fire and forget".
@@ -463,10 +507,8 @@ where
     pub events: Vec<Event>,
 }
 
-impl<T> Default for IbcBasicResponse<T>
-where
-    T: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+// Custom imlementation in order to implement it for all `T`, even if `T` is not `Default`.
+impl<T> Default for IbcBasicResponse<T> {
     fn default() -> Self {
         IbcBasicResponse {
             messages: vec![],
@@ -476,10 +518,7 @@ where
     }
 }
 
-impl<T> IbcBasicResponse<T>
-where
-    T: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+impl<T> IbcBasicResponse<T> {
     pub fn new() -> Self {
         Self::default()
     }
@@ -589,10 +628,7 @@ where
 // and not inform the calling chain).
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
 #[non_exhaustive]
-pub struct IbcReceiveResponse<T = Empty>
-where
-    T: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+pub struct IbcReceiveResponse<T = Empty> {
     /// The bytes we return to the contract that sent the packet.
     /// This may represent a success or error of exection
     pub acknowledgement: Binary,
@@ -616,10 +652,8 @@ where
     pub events: Vec<Event>,
 }
 
-impl<T> Default for IbcReceiveResponse<T>
-where
-    T: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+// Custom imlementation in order to implement it for all `T`, even if `T` is not `Default`.
+impl<T> Default for IbcReceiveResponse<T> {
     fn default() -> Self {
         IbcReceiveResponse {
             acknowledgement: Binary(vec![]),
@@ -630,10 +664,7 @@ where
     }
 }
 
-impl<T> IbcReceiveResponse<T>
-where
-    T: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+impl<T> IbcReceiveResponse<T> {
     pub fn new() -> Self {
         Self::default()
     }
@@ -786,6 +817,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::eq_op)]
     fn ibc_timeout_block_ord() {
         let epoch1a = IbcTimeoutBlock {
             revision: 1,
