@@ -1,13 +1,14 @@
 use forward_ref::{forward_ref_binop, forward_ref_op_assign};
 use schemars::JsonSchema;
 use serde::{de, ser, Deserialize, Deserializer, Serialize};
-use std::convert::{TryFrom, TryInto};
 use std::fmt::{self};
 use std::ops::{
     Add, AddAssign, Div, DivAssign, Mul, MulAssign, Rem, RemAssign, Shr, ShrAssign, Sub, SubAssign,
 };
 
-use crate::errors::{DivideByZeroError, OverflowError, OverflowOperation, StdError};
+use crate::errors::{
+    CheckedMultiplyRatioError, DivideByZeroError, OverflowError, OverflowOperation, StdError,
+};
 use crate::Uint128;
 
 /// A thin wrapper around u64 that is using strings for JSON encoding/decoding,
@@ -59,7 +60,7 @@ impl Uint64 {
         self.0.to_le_bytes()
     }
 
-    pub fn is_zero(&self) -> bool {
+    pub const fn is_zero(&self) -> bool {
         self.0 == 0
     }
 
@@ -67,6 +68,56 @@ impl Uint64 {
         self.0.pow(exp).into()
     }
 
+    /// Returns `self * numerator / denominator`
+    pub fn multiply_ratio<A: Into<u64>, B: Into<u64>>(
+        &self,
+        numerator: A,
+        denominator: B,
+    ) -> Uint64 {
+        match self.checked_multiply_ratio(numerator, denominator) {
+            Ok(value) => value,
+            Err(CheckedMultiplyRatioError::DivideByZero) => {
+                panic!("Denominator must not be zero")
+            }
+            Err(CheckedMultiplyRatioError::Overflow) => panic!("Multiplication overflow"),
+        }
+    }
+
+    /// Returns `self * numerator / denominator`
+    pub fn checked_multiply_ratio<A: Into<u64>, B: Into<u64>>(
+        &self,
+        numerator: A,
+        denominator: B,
+    ) -> Result<Uint64, CheckedMultiplyRatioError> {
+        let numerator = numerator.into();
+        let denominator = denominator.into();
+        if denominator == 0 {
+            return Err(CheckedMultiplyRatioError::DivideByZero);
+        }
+        match (self.full_mul(numerator) / Uint128::from(denominator)).try_into() {
+            Ok(ratio) => Ok(ratio),
+            Err(_) => Err(CheckedMultiplyRatioError::Overflow),
+        }
+    }
+
+    /// Multiplies two `Uint64`/`u64` values without overflow, producing an
+    /// [`Uint128`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use cosmwasm_std::Uint64;
+    ///
+    /// let a = Uint64::MAX;
+    /// let result = a.full_mul(2u32);
+    /// assert_eq!(result.to_string(), "36893488147419103230");
+    /// ```
+    pub fn full_mul(self, rhs: impl Into<u64>) -> Uint128 {
+        Uint128::from(self.u64())
+            .checked_mul(Uint128::from(rhs.into()))
+            .unwrap()
+    }
+
     pub fn checked_add(self, other: Self) -> Result<Self, OverflowError> {
         self.0
             .checked_add(other.0)
@@ -351,43 +402,6 @@ impl<'a> ShrAssign<&'a u32> for Uint64 {
     }
 }
 
-impl Uint64 {
-    /// Returns `self * numerator / denominator`
-    pub fn multiply_ratio<A: Into<u64>, B: Into<u64>>(
-        &self,
-        numerator: A,
-        denominator: B,
-    ) -> Uint64 {
-        let numerator = numerator.into();
-        let denominator = denominator.into();
-        if denominator == 0 {
-            panic!("Denominator must not be zero");
-        }
-
-        (self.full_mul(numerator) / Uint128::from(denominator))
-            .try_into()
-            .expect("multiplication overflow")
-    }
-
-    /// Multiplies two `Uint64`/`u64` values without overflow, producing an
-    /// [`Uint128`].
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use cosmwasm_std::Uint64;
-    ///
-    /// let a = Uint64::MAX;
-    /// let result = a.full_mul(2u32);
-    /// assert_eq!(result.to_string(), "36893488147419103230");
-    /// ```
-    pub fn full_mul(self, rhs: impl Into<u64>) -> Uint128 {
-        Uint128::from(self.u64())
-            .checked_mul(Uint128::from(rhs.into()))
-            .unwrap()
-    }
-}
-
 impl Serialize for Uint64 {
     /// Serializes as an integer string using base 10
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
@@ -710,6 +724,18 @@ mod tests {
         Uint64(500).multiply_ratio(1u64, 0u64);
     }
 
+    #[test]
+    fn uint64_checked_multiply_ratio_does_not_panic() {
+        assert_eq!(
+            Uint64(500u64).checked_multiply_ratio(1u64, 0u64),
+            Err(CheckedMultiplyRatioError::DivideByZero),
+        );
+        assert_eq!(
+            Uint64(500u64).checked_multiply_ratio(u64::MAX, 1u64),
+            Err(CheckedMultiplyRatioError::Overflow),
+        );
+    }
+
     #[test]
     fn sum_works() {
         let nums = vec![Uint64(17), Uint64(123), Uint64(540), Uint64(82)];
@@ -729,30 +755,40 @@ mod tests {
             Uint64::MAX.checked_add(Uint64(1)),
             Err(OverflowError { .. })
         ));
+        assert!(matches!(Uint64(1).checked_add(Uint64(1)), Ok(Uint64(2))));
         assert!(matches!(
             Uint64(0).checked_sub(Uint64(1)),
             Err(OverflowError { .. })
         ));
+        assert!(matches!(Uint64(2).checked_sub(Uint64(1)), Ok(Uint64(1))));
         assert!(matches!(
             Uint64::MAX.checked_mul(Uint64(2)),
             Err(OverflowError { .. })
         ));
+        assert!(matches!(Uint64(2).checked_mul(Uint64(2)), Ok(Uint64(4))));
         assert!(matches!(
             Uint64::MAX.checked_pow(2u32),
             Err(OverflowError { .. })
         ));
+        assert!(matches!(Uint64(2).checked_pow(3), Ok(Uint64(8))));
         assert!(matches!(
             Uint64::MAX.checked_div(Uint64(0)),
             Err(DivideByZeroError { .. })
         ));
+        assert!(matches!(Uint64(6).checked_div(Uint64(2)), Ok(Uint64(3))));
         assert!(matches!(
             Uint64::MAX.checked_div_euclid(Uint64(0)),
             Err(DivideByZeroError { .. })
         ));
+        assert!(matches!(
+            Uint64(6).checked_div_euclid(Uint64(2)),
+            Ok(Uint64(3)),
+        ));
         assert!(matches!(
             Uint64::MAX.checked_rem(Uint64(0)),
             Err(DivideByZeroError { .. })
         ));
+        assert!(matches!(Uint64(7).checked_rem(Uint64(2)), Ok(Uint64(1))));
 
         // saturating_*
         assert_eq!(Uint64::MAX.saturating_add(Uint64(1)), Uint64::MAX);
