@@ -2,6 +2,7 @@ use serde::de::DeserializeOwned;
 #[cfg(feature = "stargate")]
 use serde::Serialize;
 use std::collections::HashMap;
+use std::marker::PhantomData;
 
 use crate::addresses::{Addr, CanonicalAddr};
 use crate::binary::Binary;
@@ -27,25 +28,35 @@ use crate::serde::{from_slice, to_binary};
 use crate::storage::MemoryStorage;
 use crate::timestamp::Timestamp;
 use crate::traits::{Api, Querier, QuerierResult};
-use crate::types::{BlockInfo, ContractInfo, Env, MessageInfo};
+use crate::types::{BlockInfo, ContractInfo, Env, MessageInfo, TransactionInfo};
 use crate::Attribute;
 
 pub const MOCK_CONTRACT_ADDR: &str = "cosmos2contract";
 
-/// All external requirements that can be injected for unit tests.
-/// It sets the given balance for the contract itself, nothing else
-pub fn mock_dependencies(
-    contract_balance: &[Coin],
-) -> OwnedDeps<MockStorage, MockApi, MockQuerier> {
+/// Creates all external requirements that can be injected for unit tests.
+///
+/// See also [`mock_dependencies_with_balance`] and [`mock_dependencies_with_balances`]
+/// if you want to start with some initial balances.
+pub fn mock_dependencies() -> OwnedDeps<MockStorage, MockApi, MockQuerier, Empty> {
     OwnedDeps {
         storage: MockStorage::default(),
         api: MockApi::default(),
-        querier: MockQuerier::new(&[(MOCK_CONTRACT_ADDR, contract_balance)]),
+        querier: MockQuerier::default(),
+        custom_query_type: PhantomData,
     }
 }
 
+/// Creates all external requirements that can be injected for unit tests.
+///
+/// It sets the given balance for the contract itself, nothing else.
+pub fn mock_dependencies_with_balance(
+    contract_balance: &[Coin],
+) -> OwnedDeps<MockStorage, MockApi, MockQuerier, Empty> {
+    mock_dependencies_with_balances(&[(MOCK_CONTRACT_ADDR, contract_balance)])
+}
+
 /// Initializes the querier along with the mock_dependencies.
-/// Sets all balances provided (yoy must explicitly set contract balance if desired)
+/// Sets all balances provided (you must explicitly set contract balance if desired).
 pub fn mock_dependencies_with_balances(
     balances: &[(&str, &[Coin])],
 ) -> OwnedDeps<MockStorage, MockApi, MockQuerier> {
@@ -53,6 +64,7 @@ pub fn mock_dependencies_with_balances(
         storage: MockStorage::default(),
         api: MockApi::default(),
         querier: MockQuerier::new(balances),
+        custom_query_type: PhantomData,
     }
 }
 
@@ -216,6 +228,7 @@ pub fn mock_env() -> Env {
             time: Timestamp::from_nanos(1_571_797_419_879_305_533),
             chain_id: "cosmos-testnet-14002".to_string(),
         },
+        transaction: Some(TransactionInfo { index: 3 }),
         contract: ContractInfo {
             address: Addr::unchecked(MOCK_CONTRACT_ADDR),
         },
@@ -317,23 +330,27 @@ pub fn mock_ibc_packet_recv(
     my_channel_id: &str,
     data: &impl Serialize,
 ) -> StdResult<IbcPacketReceiveMsg> {
-    Ok(IbcPacketReceiveMsg::new(IbcPacket {
-        data: to_binary(data)?,
-        src: IbcEndpoint {
-            port_id: "their-port".to_string(),
-            channel_id: "channel-1234".to_string(),
+    Ok(IbcPacketReceiveMsg::new(
+        IbcPacket {
+            data: to_binary(data)?,
+            src: IbcEndpoint {
+                port_id: "their-port".to_string(),
+                channel_id: "channel-1234".to_string(),
+            },
+            dest: IbcEndpoint {
+                port_id: "our-port".to_string(),
+                channel_id: my_channel_id.into(),
+            },
+            sequence: 27,
+            timeout: IbcTimeoutBlock {
+                revision: 1,
+                height: 12345678,
+            }
+            .into(),
         },
-        dest: IbcEndpoint {
-            port_id: "our-port".to_string(),
-            channel_id: my_channel_id.into(),
-        },
-        sequence: 27,
-        timeout: IbcTimeoutBlock {
-            revision: 1,
-            height: 12345678,
-        }
-        .into(),
-    }))
+        #[cfg(feature = "ibc3")]
+        Addr::unchecked("relayer"),
+    ))
 }
 
 /// Creates a IbcPacket for testing ibc_packet_{ack,timeout}. You set a few key parameters that are
@@ -371,7 +388,12 @@ pub fn mock_ibc_packet_ack(
 ) -> StdResult<IbcPacketAckMsg> {
     let packet = mock_ibc_packet(my_channel_id, data)?;
 
-    Ok(IbcPacketAckMsg::new(ack, packet))
+    Ok(IbcPacketAckMsg::new(
+        ack,
+        packet,
+        #[cfg(feature = "ibc3")]
+        Addr::unchecked("relayer"),
+    ))
 }
 
 /// Creates a IbcPacketTimeoutMsg for testing ibc_packet_timeout. You set a few key parameters that are
@@ -382,7 +404,12 @@ pub fn mock_ibc_packet_timeout(
     my_channel_id: &str,
     data: &impl Serialize,
 ) -> StdResult<IbcPacketTimeoutMsg> {
-    mock_ibc_packet(my_channel_id, data).map(IbcPacketTimeoutMsg::new)
+    let packet = mock_ibc_packet(my_channel_id, data)?;
+    Ok(IbcPacketTimeoutMsg::new(
+        packet,
+        #[cfg(feature = "ibc3")]
+        Addr::unchecked("relayer"),
+    ))
 }
 
 /// The same type as cosmwasm-std's QuerierResult, but easier to reuse in
@@ -390,13 +417,12 @@ pub fn mock_ibc_packet_timeout(
 pub type MockQuerierCustomHandlerResult = SystemResult<ContractResult<Binary>>;
 
 /// MockQuerier holds an immutable table of bank balances
-/// TODO: also allow querying contracts
+/// and configurable handlers for Wasm queries and custom queries.
 pub struct MockQuerier<C: DeserializeOwned = Empty> {
     bank: BankQuerier,
     #[cfg(feature = "staking")]
     staking: StakingQuerier,
-    // placeholder to add support later
-    wasm: NoWasmQuerier,
+    wasm: WasmQuerier,
     /// A handler to handle custom queries. This is set to a dummy handler that
     /// always errors by default. Update it via `with_custom_handler`.
     ///
@@ -410,7 +436,7 @@ impl<C: DeserializeOwned> MockQuerier<C> {
             bank: BankQuerier::new(balances),
             #[cfg(feature = "staking")]
             staking: StakingQuerier::default(),
-            wasm: NoWasmQuerier {},
+            wasm: WasmQuerier::default(),
             // strange argument notation suggested as a workaround here: https://github.com/rust-lang/rust/issues/41078#issuecomment-294296365
             custom_handler: Box::from(|_: &_| -> MockQuerierCustomHandlerResult {
                 SystemResult::Err(SystemError::UnsupportedRequest {
@@ -439,6 +465,13 @@ impl<C: DeserializeOwned> MockQuerier<C> {
         self.staking = StakingQuerier::new(denom, validators, delegations);
     }
 
+    pub fn update_wasm<WH: 'static>(&mut self, handler: WH)
+    where
+        WH: Fn(&WasmQuery) -> QuerierResult,
+    {
+        self.wasm.update_handler(handler)
+    }
+
     pub fn with_custom_handler<CH: 'static>(mut self, handler: CH) -> Self
     where
         CH: Fn(&C) -> MockQuerierCustomHandlerResult,
@@ -448,6 +481,12 @@ impl<C: DeserializeOwned> MockQuerier<C> {
     }
 }
 
+impl Default for MockQuerier {
+    fn default() -> Self {
+        MockQuerier::new(&[])
+    }
+}
+
 impl<C: CustomQuery + DeserializeOwned> Querier for MockQuerier<C> {
     fn raw_query(&self, bin_request: &[u8]) -> QuerierResult {
         let request: QueryRequest<C> = match from_slice(bin_request) {
@@ -483,19 +522,43 @@ impl<C: CustomQuery + DeserializeOwned> MockQuerier<C> {
     }
 }
 
-#[derive(Clone, Default)]
-struct NoWasmQuerier {
-    // FIXME: actually provide a way to call out
+struct WasmQuerier {
+    /// A handler to handle Wasm queries. This is set to a dummy handler that
+    /// always errors by default. Update it via `with_custom_handler`.
+    ///
+    /// Use box to avoid the need of generic type.
+    handler: Box<dyn for<'a> Fn(&'a WasmQuery) -> QuerierResult>,
 }
 
-impl NoWasmQuerier {
+impl WasmQuerier {
+    fn new(handler: Box<dyn for<'a> Fn(&'a WasmQuery) -> QuerierResult>) -> Self {
+        Self { handler }
+    }
+
+    fn update_handler<WH: 'static>(&mut self, handler: WH)
+    where
+        WH: Fn(&WasmQuery) -> QuerierResult,
+    {
+        self.handler = Box::from(handler)
+    }
+
     fn query(&self, request: &WasmQuery) -> QuerierResult {
-        let addr = match request {
-            WasmQuery::Smart { contract_addr, .. } => contract_addr,
-            WasmQuery::Raw { contract_addr, .. } => contract_addr,
-        }
-        .clone();
-        SystemResult::Err(SystemError::NoSuchContract { addr })
+        (*self.handler)(request)
+    }
+}
+
+impl Default for WasmQuerier {
+    fn default() -> Self {
+        let handler = Box::from(|request: &WasmQuery| -> QuerierResult {
+            let addr = match request {
+                WasmQuery::Smart { contract_addr, .. } => contract_addr,
+                WasmQuery::Raw { contract_addr, .. } => contract_addr,
+                WasmQuery::ContractInfo { contract_addr, .. } => contract_addr,
+            }
+            .clone();
+            SystemResult::Err(SystemError::NoSuchContract { addr })
+        });
+        Self::new(handler)
     }
 }
 
@@ -692,10 +755,11 @@ pub fn mock_wasmd_attr(key: impl Into<String>, value: impl Into<String>) -> Attr
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{coin, coins, from_binary};
+    use crate::{coin, coins, from_binary, to_binary, ContractInfoResponse, Response};
     #[cfg(feature = "staking")]
     use crate::{Decimal, Delegation};
     use hex_literal::hex;
+    use serde::Deserialize;
 
     const SECP256K1_MSG_HASH_HEX: &str =
         "5ae8317d34d1e595e3fa7247db80c0af4320cce1116de187f8f7e2e099c0d8d0";
@@ -1262,6 +1326,158 @@ mod tests {
         assert_eq!(dels, Some(del2c));
     }
 
+    #[test]
+    fn wasm_querier_works() {
+        let mut querier = WasmQuerier::default();
+
+        let any_addr = "foo".to_string();
+
+        // Query WasmQuery::Raw
+        let system_err = querier
+            .query(&WasmQuery::Raw {
+                contract_addr: any_addr.clone(),
+                key: b"the key".into(),
+            })
+            .unwrap_err();
+        match system_err {
+            SystemError::NoSuchContract { addr } => assert_eq!(addr, any_addr),
+            err => panic!("Unexpected error: {:?}", err),
+        }
+
+        // Query WasmQuery::Smart
+        let system_err = querier
+            .query(&WasmQuery::Smart {
+                contract_addr: any_addr.clone(),
+                msg: b"{}".into(),
+            })
+            .unwrap_err();
+        match system_err {
+            SystemError::NoSuchContract { addr } => assert_eq!(addr, any_addr),
+            err => panic!("Unexpected error: {:?}", err),
+        }
+
+        // Query WasmQuery::ContractInfo
+        let system_err = querier
+            .query(&WasmQuery::ContractInfo {
+                contract_addr: any_addr.clone(),
+            })
+            .unwrap_err();
+        match system_err {
+            SystemError::NoSuchContract { addr } => assert_eq!(addr, any_addr),
+            err => panic!("Unexpected error: {:?}", err),
+        }
+
+        querier.update_handler(|request| {
+            let constract1 = Addr::unchecked("contract1");
+            let mut storage1 = HashMap::<Binary, Binary>::default();
+            storage1.insert(b"the key".into(), b"the value".into());
+
+            match request {
+                WasmQuery::Raw { contract_addr, key } => {
+                    if *contract_addr == constract1 {
+                        if let Some(value) = storage1.get(key) {
+                            SystemResult::Ok(ContractResult::Ok(value.clone()))
+                        } else {
+                            SystemResult::Ok(ContractResult::Ok(Binary::default()))
+                        }
+                    } else {
+                        SystemResult::Err(SystemError::NoSuchContract {
+                            addr: contract_addr.clone(),
+                        })
+                    }
+                }
+                WasmQuery::Smart { contract_addr, msg } => {
+                    if *contract_addr == constract1 {
+                        #[derive(Deserialize)]
+                        struct MyMsg {}
+                        let _msg: MyMsg = match from_binary(msg) {
+                            Ok(msg) => msg,
+                            Err(err) => {
+                                return SystemResult::Ok(ContractResult::Err(err.to_string()))
+                            }
+                        };
+                        let response: Response = Response::new().set_data(b"good");
+                        SystemResult::Ok(ContractResult::Ok(to_binary(&response).unwrap()))
+                    } else {
+                        SystemResult::Err(SystemError::NoSuchContract {
+                            addr: contract_addr.clone(),
+                        })
+                    }
+                }
+                WasmQuery::ContractInfo { contract_addr } => {
+                    if *contract_addr == constract1 {
+                        let response = ContractInfoResponse {
+                            code_id: 4,
+                            creator: "lalala".into(),
+                            admin: None,
+                            pinned: false,
+                            ibc_port: None,
+                        };
+                        SystemResult::Ok(ContractResult::Ok(to_binary(&response).unwrap()))
+                    } else {
+                        SystemResult::Err(SystemError::NoSuchContract {
+                            addr: contract_addr.clone(),
+                        })
+                    }
+                }
+            }
+        });
+
+        // WasmQuery::Raw
+        let result = querier.query(&WasmQuery::Raw {
+            contract_addr: "contract1".into(),
+            key: b"the key".into(),
+        });
+        match result {
+            SystemResult::Ok(ContractResult::Ok(value)) => assert_eq!(value, b"the value" as &[u8]),
+            res => panic!("Unexpected result: {:?}", res),
+        }
+        let result = querier.query(&WasmQuery::Raw {
+            contract_addr: "contract1".into(),
+            key: b"other key".into(),
+        });
+        match result {
+            SystemResult::Ok(ContractResult::Ok(value)) => assert_eq!(value, b"" as &[u8]),
+            res => panic!("Unexpected result: {:?}", res),
+        }
+
+        // WasmQuery::Smart
+        let result = querier.query(&WasmQuery::Smart {
+            contract_addr: "contract1".into(),
+            msg: b"{}".into(),
+        });
+        match result {
+            SystemResult::Ok(ContractResult::Ok(value)) => assert_eq!(
+                value,
+                br#"{"messages":[],"attributes":[],"events":[],"data":"Z29vZA=="}"# as &[u8]
+            ),
+            res => panic!("Unexpected result: {:?}", res),
+        }
+        let result = querier.query(&WasmQuery::Smart {
+            contract_addr: "contract1".into(),
+            msg: b"a broken request".into(),
+        });
+        match result {
+            SystemResult::Ok(ContractResult::Err(err)) => {
+                assert_eq!(err, "Error parsing into type cosmwasm_std::mock::tests::wasm_querier_works::{{closure}}::MyMsg: Invalid type")
+            }
+            res => panic!("Unexpected result: {:?}", res),
+        }
+
+        // WasmQuery::ContractInfo
+        let result = querier.query(&WasmQuery::ContractInfo {
+            contract_addr: "contract1".into(),
+        });
+        match result {
+            SystemResult::Ok(ContractResult::Ok(value)) => assert_eq!(
+                value,
+                br#"{"code_id":4,"creator":"lalala","admin":null,"pinned":false,"ibc_port":null}"#
+                    as &[u8]
+            ),
+            res => panic!("Unexpected result: {:?}", res),
+        }
+    }
+
     #[test]
     fn riffle_shuffle_works() {
         // Example from https://en.wikipedia.org/wiki/In_shuffle
